@@ -20,14 +20,14 @@
  */
 /*
   File:      actions.cpp
-  Version:   $Rev: 808 $
+  Version:   $Rev: 868 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   08-Dec-03, ahu: created
              30-Apr-06, Roger Larsson: Print filename if processing multiple files
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: actions.cpp 808 2006-06-01 15:09:39Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: actions.cpp 868 2006-08-26 06:38:16Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -37,19 +37,19 @@ EXIV2_RCSID("@(#) $Id: actions.cpp 808 2006-06-01 15:09:39Z ahuggel $");
 # include "exv_conf.h"
 #endif
 
+#ifndef EXV_HAVE_TIMEGM
+# include "timegm.h"
+#endif
 #include "actions.hpp"
+#include "exiv2.hpp"
 #include "image.hpp"
 #include "jpgimage.hpp"
-#include "exiv2.hpp"
 #include "utils.hpp"
 #include "types.hpp"
 #include "exif.hpp"
 #include "canonmn.hpp"
 #include "iptc.hpp"
 #include "futils.hpp"
-#ifndef EXV_HAVE_TIMEGM
-# include "timegm.h"
-#endif
 
 // + standard includes
 #include <string>
@@ -287,7 +287,7 @@ namespace Action {
         done = false;
         printLabel("Flash bias");
         if (!done) {
-            done = 0 != printTag(exifData, "Exif.CanonCs2.FlashBias");
+            done = 0 != printTag(exifData, "Exif.CanonSi.FlashBias");
         }
         if (!done) {
             done = 0 != printTag(exifData, "Exif.Panasonic.FlashBias");
@@ -317,7 +317,7 @@ namespace Action {
             done = 0 != printTag(exifData, "Exif.Photo.SubjectDistance");
         }
         if (!done) {
-            done = 0 != printTag(exifData, "Exif.CanonCs2.SubjectDistance");
+            done = 0 != printTag(exifData, "Exif.CanonSi.SubjectDistance");
         }
         std::cout << std::endl;
 
@@ -329,7 +329,7 @@ namespace Action {
             done = 0 != printTag(exifData, "Exif.Photo.ISOSpeedRatings");
         }
         if (!done) {
-            done = 0 != printTag(exifData, "Exif.CanonCs2.ISOSpeed");
+            done = 0 != printTag(exifData, "Exif.CanonSi.ISOSpeed");
         }
         if (!done) {
             done = 0 != printTag(exifData, "Exif.Nikon1.ISOSpeed");
@@ -362,7 +362,7 @@ namespace Action {
             done = 0 != printTag(exifData, "Exif.Photo.ExposureProgram");
         }
         if (!done) {
-            done = 0 != printTag(exifData, "Exif.CanonCs1.ExposureProgram");
+            done = 0 != printTag(exifData, "Exif.CanonCs.ExposureProgram");
         }
         if (!done) {
             done = 0 != printTag(exifData, "Exif.MinoltaCs7D.ExposureMode");
@@ -386,7 +386,7 @@ namespace Action {
         printLabel("Macro mode");
         done = false;
         if (!done) {
-            done = 0 != printTag(exifData, "Exif.CanonCs1.Macro");
+            done = 0 != printTag(exifData, "Exif.CanonCs.Macro");
         }
         if (!done) {
             done = 0 != printTag(exifData, "Exif.Fujifilm.Macro");
@@ -410,7 +410,7 @@ namespace Action {
         printLabel("Image quality");
         done = false;
         if (!done) {
-            done = 0 != printTag(exifData, "Exif.CanonCs1.Quality");
+            done = 0 != printTag(exifData, "Exif.CanonCs.Quality");
         }
         if (!done) {
             done = 0 != printTag(exifData, "Exif.Fujifilm.Quality");
@@ -478,7 +478,7 @@ namespace Action {
         printLabel("White balance");
         done = false;
         if (!done) {
-            done = 0 != printTag(exifData, "Exif.CanonCs2.WhiteBalance");
+            done = 0 != printTag(exifData, "Exif.CanonSi.WhiteBalance");
         }
         if (!done) {
             done = 0 != printTag(exifData, "Exif.Fujifilm.WhiteBalance");
@@ -1235,14 +1235,18 @@ namespace Action {
         Exiv2::ExifData &exifData = image_->exifData();
         Exiv2::IptcData &iptcData = image_->iptcData();
         if (modifyCmd.metadataId_ == exif) {
-            Exiv2::ExifData::iterator pos =
-                exifData.findKey(Exiv2::ExifKey(modifyCmd.key_));
-            if (pos != exifData.end()) exifData.erase(pos);
+            Exiv2::ExifData::iterator pos;
+            Exiv2::ExifKey exifKey = Exiv2::ExifKey(modifyCmd.key_);
+            while((pos = exifData.findKey(exifKey)) != exifData.end()) {
+                exifData.erase(pos);
+            }
         }
         if (modifyCmd.metadataId_ == iptc) {
-            Exiv2::IptcData::iterator pos =
-                iptcData.findKey(Exiv2::IptcKey(modifyCmd.key_));
-            if (pos != iptcData.end()) iptcData.erase(pos);
+            Exiv2::IptcData::iterator pos;
+            Exiv2::IptcKey iptcKey = Exiv2::IptcKey(modifyCmd.key_);
+            while((pos = iptcData.findKey(iptcKey)) != iptcData.end()) {
+                iptcData.erase(pos);
+            }
         }
     }
 
