@@ -22,7 +22,7 @@
   @file    actions.hpp
   @brief   Implements base class Task, TaskFactory and the various supported
            actions (derived from Task).
-  @version $Rev: 771 $
+  @version $Rev: 832 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    11-Dec-03, ahu: created
@@ -312,7 +312,7 @@ namespace Action {
     private:
         virtual Modify* clone_() const;
         //! Copy contructor needed because of AutoPtr member
-        Modify(const Modify& /*src*/) {}
+        Modify(const Modify& /*src*/) : Task() {}
 
         //! Add a metadatum according to \em modifyCmd
         void addMetadatum(const ModifyCmd& modifyCmd);
