@@ -20,14 +20,14 @@
  */
 /*
   File:      crwimage.cpp
-  Version:   $Rev: 754 $
+  Version:   $Rev: 861 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   28-Aug-05, ahu: created
 
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: crwimage.cpp 754 2006-05-05 17:38:12Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: crwimage.cpp 861 2006-08-15 14:40:34Z ahuggel $")
 
 // Define DEBUG to output debug information to std::cerr, e.g, by calling make
 // like this: make DEFS=-DDEBUG crwimage.o
@@ -73,7 +73,9 @@ namespace {
     //! Helper class to map Exif orientation values to CRW rotation degrees
     class RotationMap {
     public:
+        //! Get the orientation number for a degree value
         static uint16_t orientation(int32_t degrees);
+        //! Get the degree value for an orientation number
         static int32_t  degrees(uint16_t orientation);
     private:
         // DATA
@@ -113,7 +115,7 @@ namespace Exiv2 {
         CrwMapping(0x102a, 0x300b,   0, 0x0004, canonIfdId,  decodeArray,  encodeArray),
         CrwMapping(0x102d, 0x300b,   0, 0x0001, canonIfdId,  decodeArray,  encodeArray),
         CrwMapping(0x1033, 0x300b,   0, 0x000f, canonIfdId,  decodeArray,  encodeArray),
-        CrwMapping(0x1038, 0x300b,   0, 0x0012, canonIfdId,  decodeBasic,  encodeBasic),
+        CrwMapping(0x1038, 0x300b,   0, 0x0012, canonIfdId,  decodeArray,  encodeArray),
         CrwMapping(0x10a9, 0x300b,   0, 0x00a9, canonIfdId,  decodeBasic,  encodeBasic),
         // Mapped to Exif.Photo.ColorSpace instead (see below)
         //CrwMapping(0x10b4, 0x300b,   0, 0x00b4, canonIfdId,  decodeBasic,  encodeBasic),
@@ -236,14 +238,8 @@ namespace Exiv2 {
             throw Error(33);
         }
         clearMetadata();
+        CrwParser::decode(this, io_->mmap(), io_->size());
 
-        // Read the image into a memory buffer
-        long len = io_->size();
-        DataBuf buf(len);
-        io_->read(buf.pData_, len);
-        if (io_->error() || io_->eof()) throw Error(14);
-
-        CrwParser::decode(this, buf.pData_, buf.size_);
     } // CrwImage::readMetadata
 
     void CrwImage::writeMetadata()
@@ -343,7 +339,7 @@ namespace Exiv2 {
         doAdd(component);
     }
 
-    void CiffEntry::doAdd(AutoPtr component)
+    void CiffEntry::doAdd(AutoPtr /*component*/)
     {
         throw Error(34, "CiffEntry::add");
     } // CiffEntry::doAdd
@@ -787,7 +783,7 @@ namespace Exiv2 {
         return doAdd(crwDirs, crwTagId);
     } // CiffComponent::add
 
-    CiffComponent* CiffComponent::doAdd(CrwDirs& crwDirs, uint16_t crwTagId)
+    CiffComponent* CiffComponent::doAdd(CrwDirs& /*crwDirs*/, uint16_t /*crwTagId*/)
     {
         return 0;
     } // CiffComponent::doAdd
@@ -865,7 +861,7 @@ namespace Exiv2 {
         return doRemove(crwDirs, crwTagId);
     } // CiffComponent::remove
 
-    void CiffComponent::doRemove(CrwDirs& crwDirs, uint16_t crwTagId)
+    void CiffComponent::doRemove(CrwDirs& /*crwDirs*/, uint16_t /*crwTagId*/)
     {
         // do nothing
     } // CiffComponent::doRemove
@@ -949,7 +945,7 @@ namespace Exiv2 {
     } // CrwMap::decode0x0805
 
     void CrwMap::decode0x080a(const CiffComponent& ciffComponent,
-                              const CrwMapping*    pCrwMapping,
+                              const CrwMapping*    /*pCrwMapping*/,
                                     Image&         image,
                                     ByteOrder      byteOrder)
     {
@@ -992,9 +988,10 @@ namespace Exiv2 {
 
         IfdId ifdId = ifdIdNotSet;
         switch (pCrwMapping->tag_) {
-        case 0x0001: ifdId = canonCs1IfdId; break;
-        case 0x0004: ifdId = canonCs2IfdId; break;
-        case 0x000f: ifdId = canonCfIfdId;  break;
+        case 0x0001: ifdId = canonCsIfdId; break;
+        case 0x0004: ifdId = canonSiIfdId; break;
+        case 0x000f: ifdId = canonCfIfdId; break;
+        case 0x0012: ifdId = canonPiIfdId; break;
         }
         assert(ifdId != ifdIdNotSet);
 
@@ -1004,15 +1001,15 @@ namespace Exiv2 {
             uint16_t n = 1;
             ExifKey key(c, ifdItem);
             UShortValue value;
-            if (ifdId == canonCs1IfdId && c == 23 && ciffComponent.size() > 50) n = 3;
+            if (ifdId == canonCsIfdId && c == 23 && ciffComponent.size() > 50) n = 3;
             value.read(ciffComponent.pData() + c*2, n*2, byteOrder);
             image.exifData().add(key, &value);
-            if (ifdId == canonCs2IfdId && c == 21) aperture = value.toLong();
-            if (ifdId == canonCs2IfdId && c == 22) shutterSpeed = value.toLong();
+            if (ifdId == canonSiIfdId && c == 21) aperture = value.toLong();
+            if (ifdId == canonSiIfdId && c == 22) shutterSpeed = value.toLong();
             c += n;
         }
 
-        if (ifdId == canonCs2IfdId) {
+        if (ifdId == canonSiIfdId) {
             // Exif.Photo.FNumber
             float f = fnumber(canonEv(aperture));
             // Beware: primitive conversion algorithm
@@ -1144,8 +1141,7 @@ namespace Exiv2 {
 
     void CrwMap::encode(CiffHeader* pHead, const Image& image)
     {
-        for (const CrwMapping* cmi = crwMapping_;
-             cmi->ifdId_ != ifdIdNotSet; ++cmi) {
+        for (const CrwMapping* cmi = crwMapping_; cmi->ifdId_ != ifdIdNotSet; ++cmi) {
             if (cmi->fromExif_ != 0) {
                 cmi->fromExif_(image, cmi, pHead);
             }
@@ -1239,9 +1235,10 @@ namespace Exiv2 {
 
         IfdId ifdId = ifdIdNotSet;
         switch (pCrwMapping->tag_) {
-        case 0x0001: ifdId = canonCs1IfdId; break;
-        case 0x0004: ifdId = canonCs2IfdId; break;
-        case 0x000f: ifdId = canonCfIfdId;  break;
+        case 0x0001: ifdId = canonCsIfdId; break;
+        case 0x0004: ifdId = canonSiIfdId; break;
+        case 0x000f: ifdId = canonCfIfdId; break;
+        case 0x0012: ifdId = canonPiIfdId; break;
         }
         assert(ifdId != ifdIdNotSet);
         DataBuf buf = packIfdId(image.exifData(), ifdId, pHead->byteOrder());
@@ -1348,7 +1345,7 @@ namespace Exiv2 {
 
     Image::AutoPtr newCrwInstance(BasicIo::AutoPtr io, bool create)
     {
-        Image::AutoPtr image = Image::AutoPtr(new CrwImage(io, create));
+        Image::AutoPtr image(new CrwImage(io, create));
         if (!image->good()) {
             image.reset();
         }
@@ -1412,7 +1409,7 @@ namespace {
         { 8,   90 },
         { 8, -270 },
         // last entry
-        { 0,    0 },
+        { 0,    0 }
     };
 
     uint16_t RotationMap::orientation(int32_t degrees)
