@@ -20,13 +20,13 @@
  */
 /*
   File:      datasets.cpp
-  Version:   $Rev: 669 $
+  Version:   $Rev: 833 $
   Author(s): Brad Schick (brad) <brad@robotbattle.com>
   History:   24-Jul-04, brad: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: datasets.cpp 669 2006-01-17 14:53:57Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: datasets.cpp 833 2006-06-21 09:33:03Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -315,7 +315,7 @@ namespace Exiv2 {
     }
 
     IptcKey::IptcKey(const IptcKey& rhs)
-        : tag_(rhs.tag_), record_(rhs.record_), key_(rhs.key_)
+        : Key(rhs), tag_(rhs.tag_), record_(rhs.record_), key_(rhs.key_)
     {
     }
 
