@@ -22,13 +22,13 @@
   Abstract:  Command line program to display and manipulate image %Exif data
 
   File:      exiv2.cpp
-  Version:   $Rev: 788 $
+  Version:   $Rev: 875 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   10-Dec-03, ahu: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: exiv2.cpp 788 2006-05-21 10:12:18Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: exiv2.cpp 875 2006-09-12 15:03:34Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -903,12 +903,8 @@ namespace {
 
             value = line.substr(valStart, valEnd+1-valStart);
             std::string::size_type last = value.length()-1;
-            if (  (value[0] == '"' || value[last] == '"')
-                && value[0] != value[last]) {
-                throw Exiv2::Error(1, Exiv2::toString(num)
-                                    + ": Unbalanced quotes");
-            }
-            if (value[0] == '"') {
+            if (   (value[0] == '"' && value[last] == '"')
+                || (value[0] == '\'' && value[last] == '\'')) {
                 value = value.substr(1, value.length()-2);
             }
         }
