@@ -20,14 +20,14 @@
  */
 /*
   File:      ifd.cpp
-  Version:   $Rev: 763 $
+  Version:   $Rev: 859 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   26-Jan-04, ahu: created
              11-Feb-04, ahu: isolated as a component
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: ifd.cpp 763 2006-05-09 19:42:23Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: ifd.cpp 859 2006-08-13 14:55:39Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -193,17 +193,25 @@ namespace Exiv2 {
 
     void Entry::setDataAreaOffsets(uint32_t offset, ByteOrder byteOrder)
     {
+        // Hack: Do not require offsets to start from 0, except for rationals
+        uint16_t  fusOffset = 0;
+        uint32_t  fulOffset = 0;
+        int16_t   fsOffset  = 0;
+        int32_t   flOffset  = 0;
+
         for (uint32_t i = 0; i < count(); ++i) {
             byte* buf = pData_ + i * typeSize();
             switch(TypeId(type())) {
             case unsignedShort: {
-                uint16_t d = getUShort(buf, byteOrder);
+                if (i == 0) fusOffset = getUShort(buf, byteOrder);
+                uint16_t d = getUShort(buf, byteOrder) - fusOffset;
                 if (d + offset > 0xffff) throw Error(26);
                 us2Data(buf, d + static_cast<uint16_t>(offset), byteOrder);
                 break;
             }
             case unsignedLong: {
-                ul2Data(buf, getULong(buf, byteOrder) + offset, byteOrder);
+                if (i == 0) fulOffset = getULong(buf, byteOrder);
+                ul2Data(buf, getULong(buf, byteOrder) - fulOffset + offset, byteOrder);
                 break;
             }
             case unsignedRational: {
@@ -213,13 +221,15 @@ namespace Exiv2 {
                 break;
             }
             case signedShort: {
-                int16_t d = getShort(buf, byteOrder);
+                if (i == 0) fsOffset = getShort(buf, byteOrder);
+                int16_t d = getShort(buf, byteOrder) - fsOffset;
                 if (d + static_cast<int32_t>(offset) > 0xffff) throw Error(26);
                 s2Data(buf, d + static_cast<int16_t>(offset), byteOrder);
                 break;
             }
             case signedLong: {
-                int32_t d = getLong(buf, byteOrder);
+                if (i == 0) flOffset = getLong(buf, byteOrder);
+                int32_t d = getLong(buf, byteOrder) - flOffset;
                 l2Data(buf, d + static_cast<int32_t>(offset), byteOrder);
                 break;
             }
@@ -308,7 +318,13 @@ namespace Exiv2 {
         long o = start;
         Ifd::PreEntries preEntries;
 
-        if (len < o + 2) rc = 6;
+        if (o < 0 || len < o + 2) {
+#ifndef SUPPRESS_WARNINGS
+            std::cerr << "Error: " << ExifTags::ifdName(ifdId_)
+                      << " lies outside of the IFD memory buffer.\n";
+#endif
+            rc = 6;
+        }
         if (rc == 0) {
             offset_ = start - shift;
             int n = getUShort(buf + o, byteOrder);
@@ -352,6 +368,14 @@ namespace Exiv2 {
                     pNext_ = const_cast<byte*>(buf + o);
                 }
                 next_ = getULong(buf + o, byteOrder);
+                if (   static_cast<long>(next_) + shift < 0 
+                    || static_cast<long>(next_) + shift >= len) {
+#ifndef SUPPRESS_WARNINGS
+                    std::cerr << "Warning: " << ExifTags::ifdName(ifdId_)
+                              << ": Pointer to next IFD is out of bounds; ignored.\n";
+#endif
+                    next_ = 0;
+                }
             }
         }
         // Set the offset of the first data entry outside of the IFD.
@@ -679,7 +703,7 @@ namespace Exiv2 {
             std::ostringstream offset;
             if (i->size() > 4) {
                 offset << " 0x" << std::setw(8) << std::setfill('0')
-                       << std::hex << std::right << i->offset();
+                       << std::hex << std::right << static_cast<int32_t>(i->offset());
             }
             else {
                 const byte* data = i->data();
