@@ -22,7 +22,7 @@
   Abstract : Tester application for image file handling
 
   File     : metacopy.cpp
-  Version  : $Rev: 669 $
+  Version  : $Rev: 832 $
   Author(s): Brad Schick (brad) <brad@robotbattle.com>
   History  : 13-Jul-04, brad: created
  */
@@ -92,7 +92,7 @@ catch (Exiv2::AnyError& e) {
 }
 }
 
-int Params::option(int opt, const std::string& optarg, int optopt)
+int Params::option(int opt, const std::string& /*optarg*/, int optopt)
 {
     int rc = 0;
     switch (opt) {
