@@ -20,7 +20,7 @@
  */
 /*
   File:      olympusmn.cpp
-  Version:   $Rev: 783 $
+  Version:   $Rev: 832 $
   Author(s): Will Stokes (wuz) <wstokes@gmail.com>
              Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   10-Mar-05, wuz: created
@@ -29,7 +29,7 @@
 
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: olympusmn.cpp 783 2006-05-20 06:55:39Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: olympusmn.cpp 832 2006-06-21 09:16:36Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -179,8 +179,8 @@ namespace Exiv2 {
     }
 
     int OlympusMakerNote::readHeader(const byte* buf,
-                                   long len,
-                                   ByteOrder byteOrder)
+                                     long        len,
+                                     ByteOrder   /*byteOrder*/)
     {
         if (len < 8) return 1;
 
@@ -211,7 +211,7 @@ namespace Exiv2 {
 
     OlympusMakerNote* OlympusMakerNote::create_(bool alloc) const
     {
-        AutoPtr makerNote = AutoPtr(new OlympusMakerNote(alloc));
+        AutoPtr makerNote(new OlympusMakerNote(alloc));
         assert(makerNote.get() != 0);
         makerNote->readHeader(header_.pData_, header_.size_, byteOrder_);
         return makerNote.release();
@@ -274,11 +274,11 @@ namespace Exiv2 {
 // *****************************************************************************
 // free functions
 
-    MakerNote::AutoPtr createOlympusMakerNote(bool alloc,
-                                              const byte* buf,
-                                              long len,
-                                              ByteOrder byteOrder,
-                                              long offset)
+    MakerNote::AutoPtr createOlympusMakerNote(bool        alloc,
+                                              const byte* /*buf*/,
+                                              long        /*len*/,
+                                              ByteOrder   /*byteOrder*/,
+                                              long        /*offset*/)
     {
         return MakerNote::AutoPtr(new OlympusMakerNote(alloc));
     }
