@@ -20,14 +20,14 @@
  */
 /*
   File:      panasonicmn.cpp
-  Version:   $Rev: 783 $
+  Version:   $Rev: 832 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   11-Jun-04, ahu: created
   Credits:   See header file
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: panasonicmn.cpp 783 2006-05-20 06:55:39Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: panasonicmn.cpp 832 2006-06-21 09:16:36Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -196,8 +196,8 @@ namespace Exiv2 {
     }
 
     int PanasonicMakerNote::readHeader(const byte* buf,
-                                       long len,
-                                       ByteOrder byteOrder)
+                                       long        len,
+                                       ByteOrder   /*byteOrder*/)
     {
         if (len < 12) return 1;
 
@@ -227,7 +227,7 @@ namespace Exiv2 {
 
     PanasonicMakerNote* PanasonicMakerNote::create_(bool alloc) const
     {
-        AutoPtr makerNote = AutoPtr(new PanasonicMakerNote(alloc));
+        AutoPtr makerNote(new PanasonicMakerNote(alloc));
         assert(makerNote.get() != 0);
         makerNote->readHeader(header_.pData_, header_.size_, byteOrder_);
         return makerNote.release();
@@ -272,11 +272,11 @@ namespace Exiv2 {
 // *****************************************************************************
 // free functions
 
-    MakerNote::AutoPtr createPanasonicMakerNote(bool alloc,
-                                           const byte* buf,
-                                           long len,
-                                           ByteOrder byteOrder,
-                                           long offset)
+    MakerNote::AutoPtr createPanasonicMakerNote(bool        alloc,
+                                                const byte* /*buf*/,
+                                                long        /*len*/,
+                                                ByteOrder   /*byteOrder*/,
+                                                long        /*offset*/)
     {
         return MakerNote::AutoPtr(new PanasonicMakerNote(alloc));
     }
