@@ -20,7 +20,7 @@
  */
 /*
   File:      sigmamn.cpp
-  Version:   $Rev: 669 $
+  Version:   $Rev: 832 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   02-Apr-04, ahu: created
   Credits:   Sigma and Foveon MakerNote implemented according to the specification
@@ -29,7 +29,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: sigmamn.cpp 669 2006-01-17 14:53:57Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: sigmamn.cpp 832 2006-06-21 09:16:36Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -104,8 +104,8 @@ namespace Exiv2 {
     }
 
     int SigmaMakerNote::readHeader(const byte* buf,
-                                   long len,
-                                   ByteOrder byteOrder)
+                                   long        len,
+                                   ByteOrder   /*byteOrder*/)
     {
         if (len < 10) return 1;
 
@@ -140,7 +140,7 @@ namespace Exiv2 {
 
     SigmaMakerNote* SigmaMakerNote::create_(bool alloc) const
     {
-        AutoPtr makerNote = AutoPtr(new SigmaMakerNote(alloc));
+        AutoPtr makerNote(new SigmaMakerNote(alloc));
         assert(makerNote.get() != 0);
         makerNote->readHeader(header_.pData_, header_.size_, byteOrder_);
         return makerNote.release();
@@ -196,11 +196,11 @@ namespace Exiv2 {
 // *****************************************************************************
 // free functions
 
-    MakerNote::AutoPtr createSigmaMakerNote(bool alloc,
-                                            const byte* buf,
-                                            long len,
-                                            ByteOrder byteOrder,
-                                            long offset)
+    MakerNote::AutoPtr createSigmaMakerNote(bool        alloc,
+                                            const byte* /*buf*/,
+                                            long        /*len*/,
+                                            ByteOrder   /*byteOrder*/,
+                                            long        /*offset*/)
     {
         return MakerNote::AutoPtr(new SigmaMakerNote(alloc));
     }
