@@ -20,13 +20,13 @@
  */
 /*
   File:      sonymn.cpp
-  Version:   $Rev: 669 $
+  Version:   $Rev: 832 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   18-Apr-05, ahu: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: sonymn.cpp 669 2006-01-17 14:53:57Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: sonymn.cpp 832 2006-06-21 09:16:36Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -86,8 +86,8 @@ namespace Exiv2 {
     }
 
     int SonyMakerNote::readHeader(const byte* buf,
-                                   long len,
-                                   ByteOrder byteOrder)
+                                  long        len,
+                                  ByteOrder   /*byteOrder*/)
     {
         if (len < 12) return 1;
         header_.alloc(12);
@@ -116,7 +116,7 @@ namespace Exiv2 {
 
     SonyMakerNote* SonyMakerNote::create_(bool alloc) const
     {
-        AutoPtr makerNote = AutoPtr(new SonyMakerNote(alloc));
+        AutoPtr makerNote(new SonyMakerNote(alloc));
         assert(makerNote.get() != 0);
         makerNote->readHeader(header_.pData_, header_.size_, byteOrder_);
         return makerNote.release();
@@ -135,11 +135,11 @@ namespace Exiv2 {
 // *****************************************************************************
 // free functions
 
-    MakerNote::AutoPtr createSonyMakerNote(bool alloc,
-                                            const byte* buf,
-                                            long len,
-                                            ByteOrder byteOrder,
-                                            long offset)
+    MakerNote::AutoPtr createSonyMakerNote(bool        alloc,
+                                           const byte* /*buf*/,
+                                           long        /*len*/,
+                                           ByteOrder   /*byteOrder*/,
+                                           long        /*offset*/)
     {
         return MakerNote::AutoPtr(new SonyMakerNote(alloc));
     }
