@@ -21,7 +21,7 @@
 /*!
   @file    tags.hpp
   @brief   Exif tag and type information
-  @version $Rev: 808 $
+  @version $Rev: 872 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    15-Jan-04, ahu: created<BR>
@@ -124,6 +124,15 @@ namespace Exiv2 {
         bool operator==(long key) const { return val_ == key; }
     }; // struct TagDetails
 
+    /*!
+      @brief Helper structure for lookup tables for translations of bitmask
+             values to human readable labels.
+     */
+    struct TagDetailsBitmask {
+        uint32_t mask_;                         //!< Bitmask value
+        char* label_;                           //!< Description of the tag value
+    }; // struct TagDetailsBitmask
+
     /*!
       @brief Generic print function to translate a long value to a description
              by looking up a reference table.
@@ -144,6 +153,35 @@ namespace Exiv2 {
 //! Shortcut for the printTag template which requires typing the array name only once.
 #define EXV_PRINT_TAG(array) printTag<EXV_COUNTOF(array), array>
 
+    /*!
+      @brief Generic print function to translate a long value to a description
+             by looking up bitmasks in a reference table.
+     */
+    template <int N, const TagDetailsBitmask (&array)[N]>
+    std::ostream& printTagBitmask(std::ostream& os, const Value& value)
+    {
+        const uint32_t val = static_cast<uint32_t>(value.toLong());
+        bool sep = false;
+        for (int i = 0; i < N; i++) {
+            // *& acrobatics is a workaround for a MSVC 7.1 bug
+            const TagDetailsBitmask* td = *(&array) + i;
+
+            if (val & td->mask_) {
+                if (sep) {
+                    os << ", " << td->label_;
+                }
+                else {
+                    os << td->label_;
+                    sep = true;
+                }
+            }
+        }
+        return os;
+    }
+
+//! Shortcut for the printTagBitmask template which requires typing the array name only once.
+#define EXV_PRINT_TAG_BITMASK(array) printTagBitmask<EXV_COUNTOF(array), array>
+
     //! Container for Exif tag information. Implemented as a static class.
     class ExifTags {
         //! Prevent construction: not implemented.
@@ -394,25 +432,13 @@ namespace Exiv2 {
     std::ostream& printLong(std::ostream& os, const Value& value);
     //! Print a Rational or URational value in floating point format
     std::ostream& printFloat(std::ostream& os, const Value& value);
-    //! Print the unit for measuring X and Y resolution
-    std::ostream& printUnit(std::ostream& os, const Value& value);
 
-    //! Print the compression scheme used for the image data
-    std::ostream& print0x0103(std::ostream& os, const Value& value);
-    //! Print the pixel composition
-    std::ostream& print0x0106(std::ostream& os, const Value& value);
-    //! Print the orientation
-    std::ostream& print0x0112(std::ostream& os, const Value& value);
-    //! Print the YCbCrPositioning
-    std::ostream& print0x0213(std::ostream& os, const Value& value);
     //! Print the copyright
     std::ostream& print0x8298(std::ostream& os, const Value& value);
     //! Print the exposure time
     std::ostream& print0x829a(std::ostream& os, const Value& value);
     //! Print the f-number
     std::ostream& print0x829d(std::ostream& os, const Value& value);
-    //! Print the exposure mode
-    std::ostream& print0x8822(std::ostream& os, const Value& value);
     //! Print ISO speed ratings
     std::ostream& print0x8827(std::ostream& os, const Value& value);
     //! Print components configuration specific to compressed data
@@ -425,44 +451,14 @@ namespace Exiv2 {
     std::ostream& print0x9204(std::ostream& os, const Value& value);
     //! Print the subject distance
     std::ostream& print0x9206(std::ostream& os, const Value& value);
-    //! Print the metering mode
-    std::ostream& print0x9207(std::ostream& os, const Value& value);
-    //! Print the light source
-    std::ostream& print0x9208(std::ostream& os, const Value& value);
-    //! Print the flash status
-    std::ostream& print0x9209(std::ostream& os, const Value& value);
     //! Print the actual focal length of the lens
     std::ostream& print0x920a(std::ostream& os, const Value& value);
     //! Print the user comment
     std::ostream& print0x9286(std::ostream& os, const Value& value);
-    //! Print color space information
-    std::ostream& print0xa001(std::ostream& os, const Value& value);
-    //! Print info on image sensor type on the camera or input device
-    std::ostream& print0xa217(std::ostream& os, const Value& value);
-    //! Print file source
-    std::ostream& print0xa300(std::ostream& os, const Value& value);
-    //! Print scene type
-    std::ostream& print0xa301(std::ostream& os, const Value& value);
-    //! Print the exposure mode
-    std::ostream& print0xa402(std::ostream& os, const Value& value);
-    //! Print white balance information
-    std::ostream& print0xa403(std::ostream& os, const Value& value);
     //! Print digital zoom ratio
     std::ostream& print0xa404(std::ostream& os, const Value& value);
     //! Print 35mm equivalent focal length
     std::ostream& print0xa405(std::ostream& os, const Value& value);
-    //! Print scene capture type
-    std::ostream& print0xa406(std::ostream& os, const Value& value);
-    //! Print overall image gain adjustment
-    std::ostream& print0xa407(std::ostream& os, const Value& value);
-    //! Print contract adjustment
-    std::ostream& print0xa408(std::ostream& os, const Value& value);
-    //! Print saturation adjustment
-    std::ostream& print0xa409(std::ostream& os, const Value& value);
-    //! Print sharpness adjustment
-    std::ostream& print0xa40a(std::ostream& os, const Value& value);
-    //! Print subject distance range
-    std::ostream& print0xa40c(std::ostream& os, const Value& value);
     //@}
 
     //! Calculate F number from an APEX aperture value
