@@ -20,13 +20,13 @@
  */
 /*
   File:      tiffvisitor.cpp
-  Version:   $Rev: 808 $
+  Version:   $Rev: 854 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   11-Apr-06, ahu: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: tiffvisitor.cpp 808 2006-06-01 15:09:39Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: tiffvisitor.cpp 854 2006-08-02 14:16:10Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -38,6 +38,7 @@ EXIV2_RCSID("@(#) $Id: tiffvisitor.cpp 808 2006-06-01 15:09:39Z ahuggel $");
 
 #include "tiffvisitor.hpp"
 #include "tiffcomposite.hpp"
+#include "tiffparser.hpp"
 #include "makernote2.hpp"
 #include "exif.hpp"
 #include "iptc.hpp"
@@ -55,24 +56,6 @@ EXIV2_RCSID("@(#) $Id: tiffvisitor.cpp 808 2006-06-01 15:09:39Z ahuggel $");
 // class member definitions
 namespace Exiv2 {
 
-    // TIFF Decoder table for special decoding requirements
-    const TiffDecoderInfo TiffMetadataDecoder::tiffDecoderInfo_[] = {
-        { "*",       Tag::all, Group::ignr,    0 }, // Do not decode tags with group == Group::ignr
-        { "OLYMPUS",   0x0100, Group::olympmn, &TiffMetadataDecoder::decodeOlympThumb },
-        { "*",         0x014a, Group::ifd0,    0 }, // Todo: Controversial, causes problems with Exiftool
-        { "*",       Tag::all, Group::sub0_0,  &TiffMetadataDecoder::decodeSubIfd     },
-        { "*",       Tag::all, Group::sub0_1,  &TiffMetadataDecoder::decodeSubIfd     },
-        { "*",         0x8649, Group::ifd0,    &TiffMetadataDecoder::decodeIrbIptc    }
-    };
-
-    bool TiffDecoderInfo::operator==(const TiffDecoderInfo::Key& key) const
-    {
-        std::string make(make_);
-        return    ("*" == make || make == key.m_.substr(0, make.length()))
-               && (Tag::all == extendedTag_ || key.e_ == extendedTag_)
-               && key.g_ == group_;
-    }
-
     void TiffFinder::init(uint16_t tag, uint16_t group)
     {
         tag_ = tag;
@@ -135,8 +118,12 @@ namespace Exiv2 {
 
     TiffMetadataDecoder::TiffMetadataDecoder(Image* pImage,
                                              TiffComponent* const pRoot,
+                                             FindDecoderFct findDecoderFct,
                                              uint32_t threshold)
-        : pImage_(pImage), pRoot_(pRoot), threshold_(threshold)
+        : pImage_(pImage), 
+          pRoot_(pRoot),
+          findDecoderFct_(findDecoderFct),
+          threshold_(threshold)
     {
         // Find camera make
         TiffFinder finder(0x010f, Group::ifd0);
@@ -162,7 +149,7 @@ namespace Exiv2 {
         decodeTiffEntry(object);
     }
 
-    void TiffMetadataDecoder::visitDirectory(TiffDirectory* object)
+    void TiffMetadataDecoder::visitDirectory(TiffDirectory* /*object*/)
     {
         // Nothing to do
     }
@@ -177,7 +164,7 @@ namespace Exiv2 {
         if (!object->mn_) decodeTiffEntry(object);
     }
 
-    void TiffMetadataDecoder::visitIfdMakernote(TiffIfdMakernote* object)
+    void TiffMetadataDecoder::visitIfdMakernote(TiffIfdMakernote* /*object*/)
     {
         // Nothing to do
     }
@@ -203,8 +190,8 @@ namespace Exiv2 {
         assert(pImage_ != 0);
         if (!object->pData()) return;
         byte const* record = 0;
-        uint16_t sizeHdr = 0;
-        uint16_t sizeData = 0;
+        uint32_t sizeHdr = 0;
+        uint32_t sizeData = 0;
         if (0 != Photoshop::locateIptcIrb(object->pData(), object->size(),
                                           &record, &sizeHdr, &sizeData)) {
             return;
@@ -252,15 +239,18 @@ namespace Exiv2 {
             groupType_[object->group()] = object->pValue()->toLong();
         }
 
-        const TiffDecoderInfo* td = find(tiffDecoderInfo_,
-            TiffDecoderInfo::Key(make_, object->tag(), object->group()));
-        if (td) {
-            // skip decoding if td->decoderFct_ == 0
-            if (td->decoderFct_) {
-                EXV_CALL_MEMBER_FN(*this, td->decoderFct_)(object);
-            }
-            return;
+        const DecoderFct decoderFct = findDecoderFct_(make_, 
+                                                      object->tag(),
+                                                      object->group());
+        // skip decoding if decoderFct == 0
+        if (decoderFct) {
+            EXV_CALL_MEMBER_FN(*this, decoderFct)(object);
         }
+    } // TiffMetadataDecoder::decodeTiffEntry
+
+    void TiffMetadataDecoder::decodeStdTiffEntry(const TiffEntryBase* object)
+    {
+        assert(object !=0);
         assert(pImage_ != 0);
         // "Normal" tag has low priority: only decode if it doesn't exist yet.
         // Todo: ExifKey should have an appropriate c'tor, it should not be
@@ -293,7 +283,7 @@ namespace Exiv2 {
 
     } // TiffMetadataDecoder::addExifTag
 
-    void TiffMetadataDecoder::visitArrayEntry(TiffArrayEntry* object)
+    void TiffMetadataDecoder::visitArrayEntry(TiffArrayEntry* /*object*/)
     {
         // Nothing to do
     }
@@ -359,7 +349,7 @@ namespace Exiv2 {
         }
     } // TiffPrinter::visitDirectoryNext
 
-    void TiffPrinter::visitDirectoryEnd(TiffDirectory* object)
+    void TiffPrinter::visitDirectoryEnd(TiffDirectory* /*object*/)
     {
         // Nothing to do
     } // TiffPrinter::visitDirectoryEnd
@@ -376,7 +366,7 @@ namespace Exiv2 {
         else os_ << prefix() << "Makernote ";
     } // TiffPrinter::visitMnEntry
 
-    void TiffPrinter::visitIfdMakernote(TiffIfdMakernote* object)
+    void TiffPrinter::visitIfdMakernote(TiffIfdMakernote* /*object*/)
     {
         // Nothing to do
     } // TiffPrinter::visitIfdMakernote
@@ -493,22 +483,8 @@ namespace Exiv2 {
         pRoot_->accept(finder);
         TiffEntryBase* te = dynamic_cast<TiffEntryBase*>(finder.result());
         if (te && te->pValue()) {
-            long size = te->pValue()->toLong();
-            long offset = object->pValue()->toLong();
-            if (baseOffset() + offset + size <= size_) {
-                object->pValue_->setDataArea(pData_ + baseOffset() + offset, size);
-            }
-#ifndef SUPPRESS_WARNINGS
-            else {
-                std::cerr << "Warning: "
-                          << "Directory " << object->groupName()
-                          << ", entry 0x" << std::setw(4)
-                          << std::setfill('0') << std::hex << object->tag()
-                          << " Data area exceeds data buffer, ignoring it.\n";
-            }
-#endif
+            setDataArea(object, te->pValue());
         }
-
     }
 
     void TiffReader::visitSizeEntry(TiffSizeEntry* object)
@@ -520,22 +496,48 @@ namespace Exiv2 {
         pRoot_->accept(finder);
         TiffEntryBase* te = dynamic_cast<TiffEntryBase*>(finder.result());
         if (te && te->pValue()) {
-            long offset = te->pValue()->toLong();
-            long size = object->pValue()->toLong();
-            if (baseOffset() + offset + size <= size_) {
-                te->pValue_->setDataArea(pData_ + baseOffset() + offset, size);
-            }
-#ifndef SUPPRESS_WARNINGS
-            else {
-                std::cerr << "Warning: "
-                          << "Directory " << object->groupName()
-                          << ", entry 0x" << std::setw(4)
-                          << std::setfill('0') << std::hex << object->tag()
-                          << " Data area exceeds data buffer, ignoring it.\n";
-            }
-#endif
+            setDataArea(te, object->pValue());
         }
+    }
 
+    void TiffReader::setDataArea(TiffEntryBase* pOffsetEntry, const Value* pSize)
+    {
+        assert(pOffsetEntry);
+        assert(pSize);
+
+        Value* pOffset = pOffsetEntry->pValue_;
+        assert(pOffset);
+
+        long size = 0;
+        for (long i = 0; i < pSize->count(); ++i) {
+            size += pSize->toLong(i);
+        }
+        long offset = pOffset->toLong(0);
+        // Todo: Remove limitation of Jpeg writer: strips must be contiguous
+        // Until then we check: last offset + last size - first offset == size?
+        if (  pOffset->toLong(pOffset->count()-1) 
+            + pSize->toLong(pSize->count()-1)
+            - offset != size) {
+#ifndef SUPPRESS_WARNINGS
+            std::cerr << "Warning: "
+                      << "Directory " << pOffsetEntry->groupName()
+                      << ", entry 0x" << std::setw(4)
+                      << std::setfill('0') << std::hex << pOffsetEntry->tag()
+                      << " Data area is not contiguous, ignoring it.\n";
+#endif
+            return;
+        }
+        if (baseOffset() + offset + size > size_) {
+#ifndef SUPPRESS_WARNINGS
+            std::cerr << "Warning: "
+                      << "Directory " << pOffsetEntry->groupName()
+                      << ", entry 0x" << std::setw(4)
+                      << std::setfill('0') << std::hex << pOffsetEntry->tag()
+                      << " Data area exceeds data buffer, ignoring it.\n";
+#endif
+            return;
+        }
+        pOffset->setDataArea(pData_ + baseOffset() + offset, size);
     }
 
     void TiffReader::visitDirectory(TiffDirectory* object)
@@ -548,19 +550,28 @@ namespace Exiv2 {
         if (p + 2 > pLast_) {
 #ifndef SUPPRESS_WARNINGS
             std::cerr << "Error: "
-                      << "Directory " << object->groupName() << ": "
-                      << " IFD exceeds data buffer, cannot read entry count.\n";
+                      << "Directory " << object->groupName()
+                      << ": IFD exceeds data buffer, cannot read entry count.\n";
 #endif
             return;
         }
         const uint16_t n = getUShort(p, byteOrder());
         p += 2;
+        // Sanity check with an "unreasonably" large number
+        if (n > 512) {
+#ifndef SUPPRESS_WARNINGS
+            std::cerr << "Error: " 
+                      << "Directory " << object->groupName() << " with " 
+                      << n << " entries considered invalid; not read.\n";
+#endif
+            return;
+        }
         for (uint16_t i = 0; i < n; ++i) {
             if (p + 12 > pLast_) {
 #ifndef SUPPRESS_WARNINGS
                 std::cerr << "Error: "
-                          << "Directory " << object->groupName() << ": "
-                          << " IFD entry " << i
+                          << "Directory " << object->groupName()
+                          << ": IFD entry " << i
                           << " lies outside of the data buffer.\n";
 #endif
                 return;
@@ -576,8 +587,8 @@ namespace Exiv2 {
         if (p + 4 > pLast_) {
 #ifndef SUPPRESS_WARNINGS
                 std::cerr << "Error: "
-                          << "Directory " << object->groupName() << ": "
-                          << " IFD exceeds data buffer, cannot read next pointer.\n";
+                          << "Directory " << object->groupName()
+                          << ": IFD exceeds data buffer, cannot read next pointer.\n";
 #endif
                 return;
         }
@@ -598,8 +609,8 @@ namespace Exiv2 {
                 if (baseOffset() + next > size_) {
 #ifndef SUPPRESS_WARNINGS
                     std::cerr << "Error: "
-                              << "Directory " << object->groupName() << ": "
-                              << " Next pointer is out of bounds.\n";
+                              << "Directory " << object->groupName()
+                              << ": Next pointer is out of bounds.\n";
 #endif
                     return;
                 }
@@ -701,7 +712,7 @@ namespace Exiv2 {
 
     } // TiffReader::visitIfdMakernote
 
-    void TiffReader::visitIfdMakernoteEnd(TiffIfdMakernote* object)
+    void TiffReader::visitIfdMakernoteEnd(TiffIfdMakernote* /*object*/)
     {
         // Reset state (byte order, create function, offset) back to that
         // for the image
@@ -726,19 +737,30 @@ namespace Exiv2 {
         // Component already has tag
         p += 2;
         object->type_ = getUShort(p, byteOrder());
-        // todo: check type
+        long typeSize = TypeInfo::typeSize(object->typeId());
+        if (0 == typeSize) {
+#ifndef SUPPRESS_WARNINGS
+            std::cerr << "Error: Directory " << object->groupName()
+                      << ", entry 0x" << std::setw(4)
+                      << std::setfill('0') << std::hex << object->tag()
+                      << " has an invalid type:\n"
+                      << "Type = " << std::dec << object->type_
+                      << "; skipping entry.\n";
+#endif
+            return;
+        }
         p += 2;
         object->count_ = getULong(p, byteOrder());
         p += 4;
-        object->size_ = TypeInfo::typeSize(object->typeId()) * object->count();
+        object->size_ = typeSize * object->count();
         object->offset_ = getULong(p, byteOrder());
         object->pData_ = p;
         if (object->size() > 4) {
             if (baseOffset() + object->offset() >= size_) {
 #ifndef SUPPRESS_WARNINGS
                 std::cerr << "Error: Offset of "
-                          << "directory " << object->groupName() << ", "
-                          << " entry 0x" << std::setw(4)
+                          << "directory " << object->groupName()
+                          << ", entry 0x" << std::setw(4)
                           << std::setfill('0') << std::hex << object->tag()
                           << " is out of bounds:\n"
                           << "Offset = 0x" << std::setw(8)
@@ -751,7 +773,7 @@ namespace Exiv2 {
                 return;
             }
             object->pData_ = pData_ + baseOffset() + object->offset();
-            if (object->pData() + object->size() > pLast_) {
+            if (object->size() > static_cast<uint32_t>(pLast_ - object->pData())) {
 #ifndef SUPPRESS_WARNINGS
                 std::cerr << "Warning: Upper boundary of data for "
                           << "directory " << object->groupName()
@@ -770,7 +792,16 @@ namespace Exiv2 {
                 // todo: adjust count_, make size_ a multiple of typeSize
             }
         }
-        Value::AutoPtr v = Value::create(object->typeId());
+        // On the fly type conversion for Exif.Photo.UserComment
+        // Todo: This should be somewhere else, maybe in a Value factory
+        // which takes a Key and Type
+        TypeId t = TypeId(object->typeId());
+        if (   object->tag()    == 0x9286 
+            && object->group()  == Group::exif 
+            && object->typeId() == undefined) {
+            t = comment;
+        }
+        Value::AutoPtr v = Value::create(t);
         if (v.get()) {
             v->read(object->pData(), object->size(), byteOrder());
             object->pValue_ = v.release();
